// Protocol Buffers - Google's data interchange format
// Copyright 2023 Google Inc.  All rights reserved.
//
// Use of this source code is governed by a BSD-style
// license that can be found in the LICENSE file or at
// https://developers.google.com/open-source/licenses/bsd

#include "google/protobuf/internal_message_util.h"

#include <gmock/gmock.h>
#include <gtest/gtest.h>
#include "google/protobuf/map_unittest.pb.h"
#include "google/protobuf/unittest.pb.h"

namespace google {
namespace protobuf {
namespace {


}  // namespace
}  // namespace protobuf
}  // namespace google
