// Protocol Buffers - Google's data interchange format
// Copyright 2008 Google Inc.  All rights reserved.
//
// Use of this source code is governed by a BSD-style
// license that can be found in the LICENSE file or at
// https://developers.google.com/open-source/licenses/bsd

#ifndef GOOGLE_PROTOBUF_STRING_MEMBER_ROBBER_H__
#define GOOGLE_PROTOBUF_STRING_MEMBER_ROBBER_H__

#include <string>
#include <type_traits>

#include "absl/log/absl_check.h"


#endif  // GOOGLE_PROTOBUF_STRING_MEMBER_ROBBER_H__
